"""This module implements factory for file path with file."""
from abc import abstractmethod
from pathlib import Path
from typing import Generic, Type, TypeVar

from fixturefilehandler.file_paths import RelativeDeployFilePath, RelativeVacateFilePath
from tests.testlibraries.parametrizer.file_states import MultipleFilePathState, ThreeFilePathState, TwoFilePathState

PATH_TARGET = Path("test.txt")
PATH_BACKUP = Path("test.txt.bak")
PATH_RESOURCE = Path("test.txt.dist")
TypeVarTwoFilesState = TypeVar("TypeVarTwoFilesState", bound=TwoFilePathState)


class AbstractFilePathWithResourceFactory(Generic[TypeVarTwoFilesState]):
    """This class implements abstract factory."""

    @staticmethod
    @abstractmethod
    def create(tmp_path, file_state: TypeVarTwoFilesState):
        """This class creates files and returns file path"""


class VacateFilePathWithFileFactory(AbstractFilePathWithResourceFactory):
    """This class implements factory for vacate file path."""

    @staticmethod
    def create(tmp_path, file_state: TwoFilePathState):
        file_path = RelativeVacateFilePath(PATH_TARGET, PATH_BACKUP, tmp_path)
        file_state.expect_target.create_file(file_path)
        file_state.expect_backup.create_file(file_path)
        return file_path


class DeployFilePathWithFileFactory(AbstractFilePathWithResourceFactory):
    """This class implements factory for deploy file path."""

    @staticmethod
    def create(tmp_path, file_state: ThreeFilePathState):
        file_path = RelativeDeployFilePath(PATH_TARGET, PATH_BACKUP, PATH_RESOURCE, tmp_path)
        file_state.expect_target.create_file(file_path)
        file_state.expect_backup.create_file(file_path)
        file_state.expect_resource.create_file(file_path)
        return file_path


class VacateFilePathWithDirectoryFactory(AbstractFilePathWithResourceFactory):
    """This class implements factory for vacate file path."""

    @staticmethod
    def create(tmp_path, file_state: TwoFilePathState):
        file_path = RelativeVacateFilePath(PATH_TARGET, PATH_BACKUP, tmp_path)
        file_state.expect_target.create_directory(file_path)
        file_state.expect_backup.create_directory(file_path)
        return file_path


class DeployFilePathWithDirectoryFactory(AbstractFilePathWithResourceFactory):
    """This class implements factory for deploy file path."""

    @staticmethod
    def create(tmp_path, file_state: ThreeFilePathState):
        file_path = RelativeDeployFilePath(PATH_TARGET, PATH_BACKUP, PATH_RESOURCE, tmp_path)
        file_state.expect_target.create_directory(file_path)
        file_state.expect_backup.create_directory(file_path)
        file_state.expect_resource.create_directory(file_path)
        return file_path


class FilePathWithFileFactory:
    """This class implements factory for file path and file."""

    @classmethod
    def create(cls, tmp_path, multi_file_state):
        """This class creates files and returns file path"""
        factory = cls._create(multi_file_state)
        return factory.create(tmp_path, multi_file_state)

    @classmethod
    def _create(cls, multi_file_state: MultipleFilePathState) -> Type[AbstractFilePathWithResourceFactory]:
        if isinstance(multi_file_state, TwoFilePathState):
            return VacateFilePathWithFileFactory
        if isinstance(multi_file_state, ThreeFilePathState):
            return DeployFilePathWithFileFactory
        raise ValueError()


class FilePathWithDirectoryFactory:
    """This class implements factory for file path and file."""

    @classmethod
    def create(cls, tmp_path, multi_file_state):
        """This class creates files and returns file path"""
        factory = cls._create(multi_file_state)
        return factory.create(tmp_path, multi_file_state)

    @classmethod
    def _create(cls, multi_file_state: MultipleFilePathState) -> Type[AbstractFilePathWithResourceFactory]:
        if isinstance(multi_file_state, TwoFilePathState):
            return VacateFilePathWithDirectoryFactory
        if isinstance(multi_file_state, ThreeFilePathState):
            return DeployFilePathWithDirectoryFactory
        raise ValueError()



"""Tests for factories module."""
from pathlib import Path

import pytest

from fixturefilehandler import ResourceFileDeployer, TargetFilePathVacator
from fixturefilehandler.factories import DeployerFactory, VacatorFactory
from fixturefilehandler.file_paths import RelativeDeployFilePath, SimpleDeployFilePath, YamlConfigFilePathBuilder


class TestVacatorFactory:
    """Tests for VacatorFactory."""

    @staticmethod
    @pytest.mark.parametrize(
        "file_path",
        [
            (SimpleDeployFilePath(Path("path_b"), Path("path_c"), Path("path_a"))),
            (RelativeDeployFilePath(Path("path_1"), Path("path_2"), Path("path_3"))),
            (YamlConfigFilePathBuilder()),
        ],
    )
    def test(file_path):
        """
        Factory creates vacator instance.
        Created handler has same file_path with argument.
        """
        vacator = VacatorFactory.create(file_path)
        assert issubclass(vacator, TargetFilePathVacator)
        # Reason: pylint bug @see https://github.com/PyCQA/pylint/issues/3167
        # pylint: disable=no-member
        assert vacator.FILE_PATH == file_path


class TestDeployerFactory:
    """Tests for DeployerFactory."""

    @staticmethod
    @pytest.mark.parametrize(
        "file_path",
        [
            (SimpleDeployFilePath(Path("path_b"), Path("path_c"), Path("path_a"))),
            (RelativeDeployFilePath(Path("path_1"), Path("path_2"), Path("path_3"))),
            (YamlConfigFilePathBuilder()),
        ],
    )
    def test(file_path):
        """
        Factory creates deployer instance.
        Created handler has same file_path with argument.
        """
        deployer = DeployerFactory.create(file_path)
        assert issubclass(deployer, ResourceFileDeployer)
        # Reason: pylint bug @see https://github.com/PyCQA/pylint/issues/3167
        # pylint: disable=no-member
        assert deployer.FILE_PATH == file_path



"""This module implements """
from typing import TypeVar

from fixturefilehandler import VacateFilePathInterface

TypeVarVacateFilePathInterface = TypeVar("TypeVarVacateFilePathInterface", bound=VacateFilePathInterface)



"""This module implements tests for file_paths module."""
import os
from pathlib import Path

import pytest

from fixturefilehandler.file_paths import (
    RelativeDeployFilePath,
    RelativeVacateFilePath,
    SimpleDeployFilePath,
    SimpleVacateFilePath,
    YamlConfigFilePathBuilder,
)


class TestSimpleVacateFilePath:
    """Tests for SimpleVacateFilePath."""

    @staticmethod
    @pytest.mark.parametrize(
        "path_target, path_backup", [(Path("path_a"), Path("path_b")), (Path("path_1"), Path("path_2"))]
    )
    def test(path_target, path_backup):
        """Each property returns appropriate path based on python execution directory."""
        config_path_builder = SimpleVacateFilePath(path_target, path_backup)
        assert config_path_builder.target == path_target
        assert config_path_builder.backup == path_backup


class TestSimpleDeployFilePath:
    """Tests for SimpleDeployFilePath."""

    @staticmethod
    @pytest.mark.parametrize(
        "path_target, path_backup, path_resource",
        [(Path("path_a"), Path("path_b"), Path("path_c")), (Path("path_1"), Path("path_2"), Path("path_3"))],
    )
    def test(path_target, path_backup, path_resource):
        """Each property returns appropriate path based on python execution directory."""
        config_path_builder = SimpleDeployFilePath(path_target, path_backup, path_resource)
        assert config_path_builder.target == path_target
        assert config_path_builder.backup == path_backup
        assert config_path_builder.resource == path_resource


class TestRelativeVacateFilePath:
    """Tests for RelativeVacateFilePath."""

    @staticmethod
    def test_default():
        """Each property returns appropriate path based on python execution directory."""
        config_path_builder = RelativeVacateFilePath(Path("path_a"), Path("path_b"),)
        assert config_path_builder.target == Path(os.getcwd()) / "path_a"
        assert config_path_builder.backup == Path(os.getcwd()) / "path_b"

    @staticmethod
    def test_customize():
        """Argument should be applied to properties."""
        config_path_builder = RelativeVacateFilePath(Path("path_1"), Path("path_2"), Path(__file__))
        assert config_path_builder.target == Path(__file__) / "path_1"
        assert config_path_builder.backup == Path(__file__) / "path_2"


class TestRelativeDeployFilePath:
    """Tests for RelativeFilePath."""

    @staticmethod
    def test_default():
        """Each property returns appropriate path based on python execution directory."""
        config_path_builder = RelativeDeployFilePath(Path("path_a"), Path("path_b"), Path("path_c"))
        assert config_path_builder.target == Path(os.getcwd()) / "path_a"
        assert config_path_builder.backup == Path(os.getcwd()) / "path_b"
        assert config_path_builder.resource == Path(os.getcwd()) / "path_c"

    @staticmethod
    def test_customize():
        """Argument should be applied to properties."""
        config_path_builder = RelativeDeployFilePath(Path("path_1"), Path("path_2"), Path("path_3"), Path(__file__))
        assert config_path_builder.target == Path(__file__) / "path_1"
        assert config_path_builder.backup == Path(__file__) / "path_2"
        assert config_path_builder.resource == Path(__file__) / "path_3"


class TestYamlConfigFilePathBuilder:
    """Tests for YamlConfigFilePathBuilder."""

    @staticmethod
    def test_default():
        """Each property returns appropriate path based on python execution directory."""
        config_path_builder = YamlConfigFilePathBuilder()
        assert config_path_builder.target == Path(os.getcwd()) / "config.yml"
        assert config_path_builder.backup == Path(os.getcwd()) / "config.yml.bak"
        assert config_path_builder.resource == Path(os.getcwd()) / "tests" / "config.yml.dist"

    @staticmethod
    def test_specify_target_directory():
        """Each property returns appropriate path based on specific directory."""
        config_path_builder = YamlConfigFilePathBuilder(path_target_directory=Path(__file__))
        assert config_path_builder.target == Path(__file__) / "config.yml"
        assert config_path_builder.backup == Path(__file__) / "config.yml.bak"
        assert config_path_builder.resource == Path(__file__) / "tests" / "config.yml.dist"

    @staticmethod
    def test_customize():
        """Argument should be applied to properties."""
        config_path_builder = YamlConfigFilePathBuilder(
            Path(__file__),
            Path("test/testresources"),
            Path("config_a.yml"),
            Path("config_backup.yml"),
            Path("config_b.yml"),
        )
        assert config_path_builder.target == Path(__file__) / "config_a.yml"
        assert config_path_builder.backup == Path(__file__) / "config_backup.yml"
        assert config_path_builder.resource == Path(__file__) / "test" / "testresources" / "config_b.yml"



"""Tests for fixturefilehandler.__init__ module."""

import pytest

from fixturefilehandler.file_paths import VacateFilePathInterface
from tests.testlibraries.parametrizer.action_executors import Action as Act
from tests.testlibraries.parametrizer.file_path_with_resource_factories import (
    FilePathWithDirectoryFactory,
    FilePathWithFileFactory,
)
from tests.testlibraries.parametrizer.file_states import FilePathState as FPS  # noqa
from tests.testlibraries.parametrizer.file_states import MultipleFilePathState
from tests.testlibraries.parametrizer.file_states import ThreeFilePathState as ThS
from tests.testlibraries.parametrizer.file_states import TwoFilePathState as TwS

# pylint: disable=line-too-long,bad-whitespace
PARAMS = [
    # TestFixtureFileHandler
    # test_vacate_target_if_exist
    (TwS(FPS.EXISTING, FPS.NOT_EXIST), Act.VACATE_TARGET_IF_EXIST, TwS(FPS.NOT_EXIST, FPS.EXISTING)),  # noqa E501
    (TwS(FPS.NOT_EXIST, FPS.NOT_EXIST), Act.VACATE_TARGET_IF_EXIST, TwS(FPS.NOT_EXIST, FPS.NOT_EXIST)),  # noqa E501
    (
        TwS(FPS.NOT_EXIST, FPS.BACKUP),
        Act.VACATE_TARGET_IF_EXIST_ASSERT_ERROR,
        TwS(FPS.NOT_EXIST, FPS.BACKUP),
    ),  # noqa E501
    # test_restore_backup_if_exist
    (TwS(FPS.EXISTING, FPS.BACKUP), Act.RESTORE_BACKUP_IF_EXIST, TwS(FPS.BACKUP, FPS.NOT_EXIST)),  # noqa E501
    (TwS(FPS.EXISTING, FPS.NOT_EXIST), Act.RESTORE_BACKUP_IF_EXIST, TwS(FPS.EXISTING, FPS.NOT_EXIST)),  # noqa E501
    # test_deploy_resource
    (
        ThS(FPS.EXISTING, FPS.NOT_EXIST, FPS.RESOURCE),
        Act.DEPLOY_RESOURCE,
        ThS(FPS.RESOURCE, FPS.EXISTING, FPS.RESOURCE),
    ),  # noqa E501
    (
        ThS(FPS.NOT_EXIST, FPS.NOT_EXIST, FPS.RESOURCE),
        Act.DEPLOY_RESOURCE,
        ThS(FPS.RESOURCE, FPS.NOT_EXIST, FPS.RESOURCE),
    ),  # noqa E501
    (
        ThS(FPS.NOT_EXIST, FPS.NOT_EXIST, FPS.NOT_EXIST),
        Act.DEPLOY_RESOURCE_ASSERT_ERROR,
        ThS(FPS.NOT_EXIST, FPS.NOT_EXIST, FPS.NOT_EXIST),
    ),  # noqa E501
    # TeFPSargetFilePathVacator
    # test_setup
    (TwS(FPS.EXISTING, FPS.NOT_EXIST), Act.VACATOR_SETUP, TwS(FPS.NOT_EXIST, FPS.EXISTING)),  # noqa E501
    (TwS(FPS.EXISTING, FPS.NOT_EXIST), Act.VACATOR_SETUP_WITH_ARGUMENT, TwS(FPS.NOT_EXIST, FPS.EXISTING)),  # noqa E501
    # test_teardown
    (TwS(FPS.EXISTING, FPS.BACKUP), Act.VACATOR_TEARDOWN, TwS(FPS.BACKUP, FPS.NOT_EXIST)),  # noqa E501
    (TwS(FPS.EXISTING, FPS.BACKUP), Act.VACATOR_TEARDOWN_WITH_ARGUMENT, TwS(FPS.BACKUP, FPS.NOT_EXIST)),  # noqa E501
    # TestResourceFileDeployer
    # test_setup
    (
        ThS(FPS.EXISTING, FPS.NOT_EXIST, FPS.RESOURCE),
        Act.DEPLOYER_SETUP,
        ThS(FPS.RESOURCE, FPS.EXISTING, FPS.RESOURCE),
    ),  # noqa E501
    (
        ThS(FPS.EXISTING, FPS.NOT_EXIST, FPS.RESOURCE),
        Act.DEPLOYER_SETUP_WITH_ARGUMENT,
        ThS(FPS.RESOURCE, FPS.EXISTING, FPS.RESOURCE),
    ),  # noqa E501
    # test_teardown (integration test)
    (
        ThS(FPS.EXISTING, FPS.BACKUP, FPS.RESOURCE),
        Act.DEPLOYER_TEARDOWN,
        ThS(FPS.BACKUP, FPS.NOT_EXIST, FPS.RESOURCE),
    ),  # noqa E501
    (
        ThS(FPS.EXISTING, FPS.NOT_EXIST, FPS.RESOURCE),
        Act.DEPLOYER_TEARDOWN,
        ThS(FPS.EXISTING, FPS.NOT_EXIST, FPS.RESOURCE),
    ),  # noqa E501
]


@pytest.fixture
def file_path_with_file(request, tmp_path):
    """
    This fixture prepares file path aggregate
    and files specified by param (TwoFilesState or ThreeFileState) on tmp_path.
    """
    yield FilePathWithFileFactory.create(tmp_path, request.param)


@pytest.fixture
def file_path_with_directory(request, tmp_path):
    """
    This fixture prepares file path aggregate
    and files specified by param (TwoFilesState or ThreeFileState) on tmp_path.
    """
    yield FilePathWithDirectoryFactory.create(tmp_path, request.param)


class TestFixtureFileHandlerModules:
    """Tests for fixturefilehandler modules."""

    @staticmethod
    @pytest.mark.parametrize("file_path_with_file, action, expect", PARAMS, indirect=["file_path_with_file"])
    # Reason: pytest fixture. pylint: disable=redefined-outer-name
    def test_file(file_path_with_file: VacateFilePathInterface, action: Act, expect: MultipleFilePathState):
        """Specific action should update state of each files to appropriate states."""
        action.execute(file_path_with_file)
        expect.assert_file_state(file_path_with_file)

    @staticmethod
    @pytest.mark.parametrize("file_path_with_directory, action, expect", PARAMS, indirect=["file_path_with_directory"])
    # Reason: pytest fixture. pylint: disable=redefined-outer-name
    def test_directory(file_path_with_directory: VacateFilePathInterface, action: Act, expect: MultipleFilePathState):
        """Specific action should update state of each files to appropriate states."""
        action.execute(file_path_with_directory)
        expect.assert_directory_state(file_path_with_directory)



"""This module implements handler factory"""
from typing import Type

from fixturefilehandler import (
    DeployFilePathInterface,
    ResourceFileDeployer,
    TargetFilePathVacator,
    VacateFilePathInterface,
)


class VacatorFactory:
    """This class implements create method for Vacator."""

    @staticmethod
    def create(file_path: VacateFilePathInterface) -> Type[TargetFilePathVacator]:
        """This method returns FILE_PATH override Vacator."""

        class Vacator(TargetFilePathVacator):
            """This class Vacate target file path."""

            FILE_PATH = file_path

        return Vacator


class DeployerFactory:
    """This class implements create method for Deployer."""

    @staticmethod
    def create(file_path: DeployFilePathInterface) -> Type[ResourceFileDeployer]:
        """This method returns FILE_PATH override Deployer."""

        class Deployer(ResourceFileDeployer):
            """This class deploy resource file into target file path."""

            FILE_PATH = file_path

        return Deployer



"""This module implements exceptions for this package."""


class Error(Exception):
    """
    Base class for exceptions in this module.
    @see https://docs.python.org/3/tutorial/errors.html#user-defined-exceptions
    """


class BackupAlreadyExistError(Error):
    """File or directory already exists on backup path when vacate existing file on target path."""



"""This module implements checker."""
from abc import abstractmethod
from pathlib import Path


class AbstractChecker:
    """This class implements abstruct checker."""

    def __init__(self, path: Path):
        self.path = path

    @abstractmethod
    def do_assertion(self):
        """This method assert something."""


class NotExistChecker(AbstractChecker):
    """This class implements checker in case when want to check that file not exist."""

    def do_assertion(self):
        assert not self.path.exists()


class ContentChecker(AbstractChecker):
    """This class implements checker in case when want to check that file content is as same as expected."""

    def __init__(self, path: Path, content: str):
        super().__init__(path)
        self.content = content

    def do_assertion(self):
        assert self.path.read_text() == self.content



"""This module implements """
from abc import abstractmethod
from dataclasses import dataclass
from enum import Enum
from typing import Generic, Type

from fixturefilehandler import DeployFilePathInterface, VacateFilePathInterface
from tests.testlibraries.parametrizer import TypeVarVacateFilePathInterface
from tests.testlibraries.parametrizer.file_path_state_handlers import (
    ContentBackupHandler,
    ContentExistingHandler,
    ContentResourceHandler,
    FilePathStateHandler,
    NoExistHandler,
)


class FilePathState(Enum):
    """This class implements kinds of file state."""

    NOT_EXIST: Type[FilePathStateHandler] = NoExistHandler
    EXISTING: Type[FilePathStateHandler] = ContentExistingHandler
    BACKUP: Type[FilePathStateHandler] = ContentBackupHandler
    RESOURCE: Type[FilePathStateHandler] = ContentResourceHandler

    def assert_file_state(self, path):
        """This method checks file state."""
        self.value.assert_file_state(path)

    def assert_directory_state(self, path):
        """This method checks file state."""
        self.value.assert_directory_state(path)

    def create_file(self, path):
        """This method creates file."""
        self.value.create_file(path)

    def create_directory(self, path):
        """This method creates file."""
        self.value.create_directory(path)

    @property
    def value(self) -> Type[FilePathStateHandler]:
        return super().value


class MultipleFilePathState(Generic[TypeVarVacateFilePathInterface]):
    """This class implements abstract multiple file state."""

    @abstractmethod
    def assert_file_state(self, file_path: TypeVarVacateFilePathInterface):
        """This method checks file state."""

    @abstractmethod
    def assert_directory_state(self, file_path: TypeVarVacateFilePathInterface):
        """This method checks file state."""


@dataclass
class TwoFilePathState(MultipleFilePathState):
    """This class implements states of two files."""

    expect_target: FilePathState
    expect_backup: FilePathState

    def assert_file_state(self, file_path: VacateFilePathInterface):
        self.expect_target.assert_file_state(file_path.target)
        self.expect_backup.assert_file_state(file_path.backup)

    def assert_directory_state(self, file_path: VacateFilePathInterface):
        self.expect_target.assert_directory_state(file_path.target)
        self.expect_backup.assert_directory_state(file_path.backup)


@dataclass
class ThreeFilePathState(MultipleFilePathState):
    """This class implements states of three files."""

    expect_target: FilePathState
    expect_backup: FilePathState
    expect_resource: FilePathState

    def assert_file_state(self, file_path: DeployFilePathInterface):
        self.expect_target.assert_file_state(file_path.target)
        self.expect_backup.assert_file_state(file_path.backup)
        self.expect_resource.assert_file_state(file_path.resource)

    def assert_directory_state(self, file_path: DeployFilePathInterface):
        self.expect_target.assert_directory_state(file_path.target)
        self.expect_backup.assert_directory_state(file_path.backup)
        self.expect_resource.assert_directory_state(file_path.resource)



"""This module implements file path aggregates."""
import os
from abc import abstractmethod
from pathlib import Path


class VacateFilePathInterface:
    """This interface defines properties"""

    @property
    @abstractmethod
    def target(self) -> Path:
        """This property return path to target file"""

    @property
    @abstractmethod
    def backup(self) -> Path:
        """This property return path to backup file"""


class DeployFilePathInterface(VacateFilePathInterface):
    """This interface defines properties"""

    @property
    @abstractmethod
    def resource(self) -> Path:
        """This property return path to resource file"""


class SimpleVacateFilePath(VacateFilePathInterface):
    """This class implements simple paths for vacate."""

    def __init__(self, target: Path, backup: Path):
        self._target = target
        self._backup = backup

    @property
    def target(self) -> Path:
        return self._target

    @property
    def backup(self) -> Path:
        return self._backup


class SimpleDeployFilePath(SimpleVacateFilePath, DeployFilePathInterface):
    """This class implements simple paths for deploy."""

    def __init__(self, target: Path, backup: Path, resource: Path):
        super().__init__(target, backup)
        self._resource = resource

    @property
    def resource(self) -> Path:
        return self._resource


class RelativeVacateFilePath(SimpleVacateFilePath):
    """This class implements relative paths for vacate."""

    def __init__(self, target: Path, backup: Path, base: Path = Path(os.getcwd())):
        super().__init__(target, backup)
        self._base = base

    @property
    def target(self) -> Path:
        return self._base / self._target

    @property
    def backup(self) -> Path:
        return self._base / self._backup


class RelativeDeployFilePath(SimpleDeployFilePath, RelativeVacateFilePath):
    """This class implements relative paths for deploy."""

    def __init__(self, target: Path, backup: Path, resource: Path, base: Path = Path(os.getcwd())):
        super().__init__(target, backup, resource)
        self._base = base

    @property
    def resource(self) -> Path:
        return self._base / self._resource


class YamlConfigFilePathBuilder(DeployFilePathInterface):
    """
    This class builds file path for config file.
    Default value is maybe suitable for standard directory structure of python project.
    """

    # pylint: disable=too-many-arguments
    def __init__(
        self,
        path_target_directory: Path = Path(os.getcwd()),
        path_test_directory: Path = Path("tests"),
        file_target: Path = Path("config.yml"),
        file_backup: Path = Path("config.yml.bak"),
        file_resource: Path = Path("config.yml.dist"),
    ):
        self.path_target_directory = path_target_directory
        self.path_test_directory = path_test_directory
        self.file_target = file_target
        self.file_backup = file_backup
        self.file_resource = file_resource

    @property
    def target(self) -> Path:
        return self.path_target_directory / self.file_target

    @property
    def backup(self) -> Path:
        return self.path_target_directory / self.file_backup

    @property
    def resource(self) -> Path:
        return self.path_resource_directory / self.file_resource

    @property
    def path_resource_directory(self) -> Path:
        """This property returns calculated path for resource directory."""
        return self.path_target_directory / self.path_test_directory



"""This module implements constant for scenario test."""
from pathlib import Path

from tests.testlibraries.fixturefilehandlerfortest.file_paths_for_test import (
    DeployFilePathBuilderForTest,
    VacateFilePathBuilderForTest,
)

PATH_PROJECT_HOME = Path(__file__).parent.parent.parent
PATH_YOUR_PROJECT_HOME = Path(__file__).parent.parent.parent / "yourproduct"
PATH_TEST_RESOURCES_HOME = Path(__file__).parent.parent / "testresources"
CONTENT_IN_PROJECT_TEST_RESOURCE_FILE = "content in project test resource file"
CONTENT_IN_PROJECT_TEST_RESOURCE_FILE_FOR_ADVANCED = "content in project test resources for advanced"
CONTENT_IN_TEST_RESOURCE_FILE = "content in test resource file"
FILE_PATH_VACATE = VacateFilePathBuilderForTest(
    target=Path("test.txt"),
    backup=Path("test.txt.bak"),
    backup_for_test=Path("test.txt.test.bak"),
    base=PATH_YOUR_PROJECT_HOME,
)
FILE_PATH_DEPLOY = DeployFilePathBuilderForTest(
    target=Path("test.txt"),
    backup=Path("test.txt.bak"),
    backup_for_test=Path("test.txt.test.bak"),
    resource=Path("test.txt.dist"),
    base=PATH_YOUR_PROJECT_HOME,
    base_resource=PATH_TEST_RESOURCES_HOME,
)



"""This module implements fixture file handler."""
import os
import shutil
from abc import abstractmethod
from typing import Generic, TypeVar

from fixturefilehandler.exceptions import BackupAlreadyExistError
from fixturefilehandler.file_paths import DeployFilePathInterface, VacateFilePathInterface

__version__ = "1.3.0"


class FixtureFileHandler:
    """This class implements basic static methods."""

    @staticmethod
    def vacate_target_if_exist(file_path: VacateFilePathInterface) -> None:
        """
        This method vacate target file if target file exist.
        :return: None
        """
        if file_path.backup.exists():
            raise BackupAlreadyExistError()
        if file_path.target.is_file() or file_path.target.is_dir():
            os.replace(str(file_path.target), str(file_path.backup))

    @staticmethod
    def deploy_resource(file_path: DeployFilePathInterface) -> None:
        """
        This method vacate target file if target file exist.
        :return: None
        """
        FixtureFileHandler.vacate_target_if_exist(file_path)
        if file_path.resource.is_dir():
            shutil.copytree(str(file_path.resource), str(file_path.target))
            return
        shutil.copy(str(file_path.resource), str(file_path.target))

    @staticmethod
    def restore_backup_if_exist(file_path: VacateFilePathInterface) -> None:
        """
        This method restore backup file by if exist.
        :return: None
        """
        if file_path.backup.is_file() or file_path.backup.is_dir():
            if file_path.target.is_dir():
                shutil.rmtree(str(file_path.target))
            os.replace(str(file_path.backup), str(file_path.target))


TVacateFilePathInterface = TypeVar("TVacateFilePathInterface", bound=VacateFilePathInterface)


class HandlerInterface(Generic[TVacateFilePathInterface]):
    """This class implements interface of handler."""

    FILE_PATH: TVacateFilePathInterface

    @classmethod
    @abstractmethod
    def setup(cls, file_path: TVacateFilePathInterface = None) -> None:
        """This function setup."""

    @classmethod
    def teardown(cls, file_path: TVacateFilePathInterface = None) -> None:
        """This function restore backup if exist."""
        if file_path is None:
            file_path = cls.FILE_PATH
        FixtureFileHandler.restore_backup_if_exist(file_path)


class TargetFilePathVacator(HandlerInterface):
    """This class Vacate target file path."""

    FILE_PATH: VacateFilePathInterface

    @classmethod
    def setup(cls, file_path: VacateFilePathInterface = None) -> None:
        """This function vacate target if exist."""
        if file_path is None:
            file_path = cls.FILE_PATH
        FixtureFileHandler.vacate_target_if_exist(file_path)


class ResourceFileDeployer(HandlerInterface):
    """This class Deploy resource file into target file path."""

    FILE_PATH: DeployFilePathInterface

    @classmethod
    def setup(cls, file_path: DeployFilePathInterface = None) -> None:
        """
        This method replace config file by file_path.resource.
        If file already exists, process backs up existing file.
        :return: No return
        """
        if file_path is None:
            file_path = cls.FILE_PATH
        FixtureFileHandler.deploy_resource(file_path)



"""This module implements executer for test target method."""
from abc import abstractmethod
from enum import Enum
from typing import Generic, Type

import pytest

from fixturefilehandler import (
    BackupAlreadyExistError,
    DeployFilePathInterface,
    FixtureFileHandler,
    ResourceFileDeployer,
    TargetFilePathVacator,
    VacateFilePathInterface,
)
from tests.testlibraries.parametrizer import TypeVarVacateFilePathInterface


def create_vacator_class(file_path):
    """This function returns vacator class."""

    class Vacator(TargetFilePathVacator):
        """This class implements concrete file path."""

        FILE_PATH = file_path

    return Vacator


def create_deployer_class(file_path):
    """This function returns deployer class."""

    class Deployer(ResourceFileDeployer):
        """This class implements concrete file path."""

        FILE_PATH = file_path

    return Deployer


class ActionExecutor(Generic[TypeVarVacateFilePathInterface]):
    """This class implements abstract action executor."""

    @staticmethod
    @abstractmethod
    def execute(file_path: TypeVarVacateFilePathInterface):
        """This method execute action."""


class VacateTargetIfExistExecutor(ActionExecutor[VacateFilePathInterface]):
    """This class implements action executor for vacate_target_if_exist()."""

    @staticmethod
    def execute(file_path: VacateFilePathInterface):
        FixtureFileHandler.vacate_target_if_exist(file_path)


class VacateTargetIfExistAssertErrorExecutor(ActionExecutor[VacateFilePathInterface]):
    """This class implements action executor for vacate_target_if_exist() with argument."""

    @staticmethod
    def execute(file_path: VacateFilePathInterface):
        with pytest.raises(BackupAlreadyExistError):
            FixtureFileHandler.vacate_target_if_exist(file_path)


class DeployResourceExecutor(ActionExecutor[DeployFilePathInterface]):
    """This class implements action executor for deploy_resource()."""

    @staticmethod
    def execute(file_path: DeployFilePathInterface):
        FixtureFileHandler.deploy_resource(file_path)


class DeployResourceAssertErrorExecutor(ActionExecutor[DeployFilePathInterface]):
    """This class implements action executor for deploy_resource() with argument and expect error."""

    @staticmethod
    def execute(file_path: DeployFilePathInterface):
        with pytest.raises(FileNotFoundError):
            FixtureFileHandler.deploy_resource(file_path)


class RestoreBackupIfExistExecutor(ActionExecutor[VacateFilePathInterface]):
    """This class implements action executor for restore_backup_if_exist() with argument."""

    @staticmethod
    def execute(file_path: VacateFilePathInterface):
        FixtureFileHandler.restore_backup_if_exist(file_path)


class VacatorSetupExecutor(ActionExecutor[VacateFilePathInterface]):
    """This class implements action executor for Vacator.setup()."""

    @staticmethod
    def execute(file_path: VacateFilePathInterface):
        create_vacator_class(file_path).setup()


class VacatorSetupWithArgumentExecutor(ActionExecutor[VacateFilePathInterface]):
    """This class implements action executor for Vacator.setup() with argument."""

    @staticmethod
    def execute(file_path: VacateFilePathInterface):
        TargetFilePathVacator.setup(file_path)


class VacatorTeardownExecutor(ActionExecutor[VacateFilePathInterface]):
    """This class implements action executor for Vacator.teardown()."""

    @staticmethod
    def execute(file_path: VacateFilePathInterface):
        create_vacator_class(file_path).teardown()


class VacatorTeardownWithArgumentExecutor(ActionExecutor[VacateFilePathInterface]):
    """This class implements action executor for Vacator.teardown() with argument."""

    @staticmethod
    def execute(file_path: VacateFilePathInterface):
        TargetFilePathVacator.teardown(file_path)


class DeployerSetupExecutor(ActionExecutor[DeployFilePathInterface]):
    """This class implements action executor for Deployer.setup()."""

    @staticmethod
    def execute(file_path: DeployFilePathInterface):
        create_deployer_class(file_path).setup()


class DeployerSetupWithArgumentExecutor(ActionExecutor[DeployFilePathInterface]):
    """This class implements action executor for Deployer.setup() with argument."""

    @staticmethod
    def execute(file_path: DeployFilePathInterface):
        ResourceFileDeployer.setup(file_path)


class DeployerTeardownExecutor(ActionExecutor[DeployFilePathInterface]):
    """This class implements action executor for Deployer.teardown()."""

    @staticmethod
    def execute(file_path: DeployFilePathInterface):
        create_deployer_class(file_path).teardown()


class DeployerTeardownWithArgumentExecutor(ActionExecutor[VacateFilePathInterface]):
    """This class implements action executor for Deployer.teardown() with argument."""

    @staticmethod
    def execute(file_path: VacateFilePathInterface):
        ResourceFileDeployer.teardown(file_path)


class Action(Enum):
    """This class implements kinds of action executor."""

    VACATE_TARGET_IF_EXIST = VacateTargetIfExistExecutor
    VACATE_TARGET_IF_EXIST_ASSERT_ERROR = VacateTargetIfExistAssertErrorExecutor
    DEPLOY_RESOURCE = DeployResourceExecutor
    DEPLOY_RESOURCE_ASSERT_ERROR = DeployResourceAssertErrorExecutor
    RESTORE_BACKUP_IF_EXIST = RestoreBackupIfExistExecutor
    VACATOR_SETUP = VacatorSetupExecutor
    VACATOR_SETUP_WITH_ARGUMENT = VacatorSetupWithArgumentExecutor
    VACATOR_TEARDOWN = VacatorTeardownExecutor
    VACATOR_TEARDOWN_WITH_ARGUMENT = VacatorTeardownWithArgumentExecutor
    DEPLOYER_SETUP = DeployerSetupExecutor
    DEPLOYER_SETUP_WITH_ARGUMENT = DeployerSetupWithArgumentExecutor
    DEPLOYER_TEARDOWN = DeployerTeardownExecutor
    DEPLOYER_TEARDOWN_WITH_ARGUMENT = DeployerTeardownWithArgumentExecutor

    def execute(self, file_path: VacateFilePathInterface):
        """This method be short cut for value's method."""
        self.value.execute(file_path)  # type: ignore

    @property
    def value(self) -> Type[ActionExecutor[VacateFilePathInterface]]:
        return super().value



"""This module implements file path aggregates for test."""
import os
import shutil
from abc import abstractmethod
from typing import Generic, TypeVar

from tests.testlibraries.fixturefilehandlerfortest.file_paths_for_test import (
    DeployFilePathBuilderForTest,
    VacateFilePathBuilderForTest,
)


class FixtureFileHandlerForTest:
    """This class implements basic static methods."""

    @staticmethod
    def vacate_target_if_exist_and_remove_backup_if_exist(file_path) -> None:
        """
        This method vacate target file if target file exist,
        and remove backup file if backup file exist.
        :return: None
        """
        if file_path.target.is_file():
            os.replace(str(file_path.target), str(file_path.backup_for_test))
        if file_path.backup.exists():
            os.unlink(str(file_path.backup))

    @staticmethod
    def deploy_resource(file_path) -> None:
        """
        This method vacate target file if target file exist.
        :return: None
        """
        FixtureFileHandlerForTest.vacate_target_if_exist_and_remove_backup_if_exist(file_path)
        shutil.copy(str(file_path.resource), str(file_path.target))

    @staticmethod
    def restore_target_if_backup_for_test_exist(file_path: VacateFilePathBuilderForTest) -> None:
        """
        This method restore backup file by if exist.
        :return: None
        """
        if not file_path.backup_for_test.is_file():
            return
        if file_path.target.is_file():
            os.unlink(str(file_path.target))
        os.replace(str(file_path.backup_for_test), str(file_path.target))


TVacateFilePathInterfaceForTest = TypeVar("TVacateFilePathInterfaceForTest", bound=VacateFilePathBuilderForTest)


class HandlerInterfaceForTest(Generic[TVacateFilePathInterfaceForTest]):
    """This class implements interface of handler."""

    FILE_PATH: TVacateFilePathInterfaceForTest

    @classmethod
    @abstractmethod
    def set_up(cls) -> None:
        """This function setup."""

    @classmethod
    def do_cleanups(cls) -> None:
        """This function restore backup if exist."""
        FixtureFileHandlerForTest.restore_target_if_backup_for_test_exist(cls.FILE_PATH)


class TargetFilePathVacatorForTest(HandlerInterfaceForTest):
    """This class vacates target file path."""

    FILE_PATH: VacateFilePathBuilderForTest

    @classmethod
    def set_up(cls) -> None:
        FixtureFileHandlerForTest.vacate_target_if_exist_and_remove_backup_if_exist(cls.FILE_PATH)


class FixtureFileDeployerForTest(HandlerInterfaceForTest):
    """This class deploys resource file into target file path."""

    FILE_PATH: DeployFilePathBuilderForTest

    @classmethod
    def set_up(cls) -> None:
        FixtureFileHandlerForTest.deploy_resource(cls.FILE_PATH)



"""This module implements aggregate object about file paths for vacate or deploy fixture for unit testing."""
import os
from pathlib import Path


class VacateFilePathBuilderForTest:
    """This class builds file path for vacate specific target file path."""

    def __init__(self, target: Path, backup: Path, backup_for_test: Path, base: Path = Path(os.getcwd())):
        self._target: Path = target
        self._backup: Path = backup
        self._backup_for_test: Path = backup_for_test
        self._base: Path = base

    @property
    def target(self) -> Path:
        """This property return path to target file."""
        return self._base / self._target

    @property
    def backup(self) -> Path:
        """This property return path to backup file which production code back up."""
        return self._base / self._backup

    @property
    def backup_for_test(self) -> Path:
        """This property return path to backup file which unit testing code back up."""
        return self._base / self._backup_for_test


class DeployFilePathBuilderForTest(VacateFilePathBuilderForTest):
    """This class builds file path for deploy specific resource file into specific target file path."""

    # pylint: disable=too-many-arguments
    def __init__(
        self,
        target: Path,
        backup: Path,
        backup_for_test: Path,
        resource: Path,
        base: Path = Path(os.getcwd()),
        base_resource: Path = Path(os.getcwd()) / "tests",
    ):
        super().__init__(target, backup, backup_for_test, base)
        self._resource = resource
        self._base_resource = base_resource

    @property
    def resource(self) -> Path:
        """This method return path to resource file"""
        return self._base_resource / self._resource



"""Tests for pytest scenario"""
import shutil
from pathlib import Path

import pytest

from tests.testscenarios import (
    CONTENT_IN_PROJECT_TEST_RESOURCE_FILE,
    CONTENT_IN_PROJECT_TEST_RESOURCE_FILE_FOR_ADVANCED,
    CONTENT_IN_TEST_RESOURCE_FILE,
    PATH_PROJECT_HOME,
    PATH_TEST_RESOURCES_HOME,
    PATH_YOUR_PROJECT_HOME,
)
from yourproduct.pytest_context_advanced import fixture_file_advanced  # noqa: F401 pylint: disable=unused-import
from yourproduct.pytest_context_basic import fixture_file  # noqa: F401 pylint: disable=unused-import

# pylint: disable=invalid-name
pytest_plugins = ["pytester"]


class TestConfigHandlerPytest:
    """Tests for basic pytest scenario."""

    @staticmethod
    # Reason: pytest fixture. pylint: disable=redefined-outer-name
    def test(fixture_file):  # noqa: F811
        """Config file for test should be loaded."""
        assert fixture_file.target.read_text() == CONTENT_IN_PROJECT_TEST_RESOURCE_FILE


@pytest.fixture
def directory_testdir(testdir):
    """This fixture prepares basic directory structure on testdir."""
    sample_code = (PATH_YOUR_PROJECT_HOME / "pytest_context_basic.py").read_text()
    print(
        testdir.makeconftest(
            f"import sys\nsys.path.append({str(PATH_PROJECT_HOME).__repr__()})\n" + sample_code
        ).read_text(encoding="UTF-8")
    )
    testdir.makepyfile((Path(__file__).parent.parent / "testresources/pytest_code_pytest_context.py").read_text())
    shutil.copy(
        str(PATH_YOUR_PROJECT_HOME / "testresources/test.txt.dist"),
        str(Path(testdir.mkdir("testresources")) / "test.txt.dist"),
    )
    yield testdir


@pytest.fixture
# Reason: pytest fixture. pylint: disable=redefined-outer-name
def directory_target_testdir(directory_testdir):
    """Thiws fixture prepares basic directory structure and resource file on testdir."""
    shutil.copy(str(PATH_TEST_RESOURCES_HOME / "test.txt.dist"), str(Path(directory_testdir.tmpdir) / "test.txt"))
    yield directory_testdir


class TestConfigHandlerPytestBeforeAfter:
    """Tests for basic pytest scenario including state before / after fixture works."""

    @staticmethod
    # Reason: pytest fixture. pylint: disable=redefined-outer-name
    def test_case_when_target_exist(directory_target_testdir):
        """
        Target file should be test resource before set up.
        Target file should be project test resource between set up and do cleanups.
        Target file should be test resource after do cleanups.
        """
        target = Path(directory_target_testdir.tmpdir) / "test.txt"
        assert target.read_text() == CONTENT_IN_TEST_RESOURCE_FILE
        result = directory_target_testdir.runpytest_subprocess()
        result.assert_outcomes(passed=1)
        assert target.read_text() == CONTENT_IN_TEST_RESOURCE_FILE

    @staticmethod
    # Reason: pytest fixture. pylint: disable=redefined-outer-name
    def test_case_when_target_not_exist(directory_testdir):
        """
        Target file should not exist before set up.
        Target file should be project test resource between set up and do cleanups.
        Target file should be project test resource after do cleanups.
        """
        target = Path(directory_testdir.tmpdir) / "test.txt"
        assert not target.exists()
        result = directory_testdir.runpytest_subprocess()
        result.assert_outcomes(passed=1)
        assert target.read_text() == CONTENT_IN_PROJECT_TEST_RESOURCE_FILE


class TestAdvancedConfigHandlerPytest:
    """Tests for advanced pytest scenario."""

    @staticmethod
    # Reason: pytest fixture. pylint: disable=redefined-outer-name
    def test_advanced(fixture_file_advanced):  # noqa: F811
        """Config file for test should be loaded."""
        assert fixture_file_advanced.target.read_text() == CONTENT_IN_PROJECT_TEST_RESOURCE_FILE_FOR_ADVANCED



"""Tests for unittest scenario"""
import pytest

from tests.testlibraries.fixturefilehandlerfortest.fixture_file_handlers_for_test import (
    FixtureFileDeployerForTest,
    TargetFilePathVacatorForTest,
)
from tests.testscenarios import (
    CONTENT_IN_PROJECT_TEST_RESOURCE_FILE_FOR_ADVANCED,
    CONTENT_IN_TEST_RESOURCE_FILE,
    FILE_PATH_DEPLOY,
    FILE_PATH_VACATE,
)
from yourproduct.unittest_context_advanced import AdvancedConfigurableTestCase
from yourproduct.unittest_context_basic import ConfigurableTestCase


class TestConfigHandlerUnittest(ConfigurableTestCase):
    """Tests for unittest scenario"""

    @staticmethod
    def test():
        """Config file for test should be loaded."""
        assert not FILE_PATH_DEPLOY.target.exists()


@pytest.fixture
def target_file():
    """This fixture prepares resource file."""

    class Deployer(FixtureFileDeployerForTest):
        """This class deploys resource file into target file path."""

        FILE_PATH = FILE_PATH_DEPLOY

    Deployer.set_up()
    yield
    Deployer.do_cleanups()


@pytest.fixture
def target_file_empty():
    """This fixture prepares target file path empty."""

    class Vacator(TargetFilePathVacatorForTest):
        """This class vacates resource file into target file path."""

        FILE_PATH = FILE_PATH_VACATE

    Vacator.set_up()
    yield
    Vacator.do_cleanups()


class TestConfigHandlerUnittestBeforeAfter:
    """Tests for before / after states of ConfigHandler implemented by unittest."""

    # Reason: pytest fixture. pylint: disable=unused-argument, redefined-outer-name
    def test_case_when_target_exist(self, target_file):
        """
        Target file should be test resource before set up.
        Target file should not exist between set up and do cleanups.
        Target file should be test resource after do cleanups.
        """
        assert FILE_PATH_DEPLOY.target.read_text() == CONTENT_IN_TEST_RESOURCE_FILE
        self._set_up_by_unittest()
        assert not FILE_PATH_DEPLOY.target.exists()
        self._do_cleanups_by_unittest()
        assert FILE_PATH_DEPLOY.target.read_text() == CONTENT_IN_TEST_RESOURCE_FILE

    # Reason: pytest fixture. pylint: disable=unused-argument, redefined-outer-name
    def test_case_when_target_not_exist(self, target_file_empty):
        """
        Target file should not exist anytime.
        """
        assert not FILE_PATH_VACATE.target.exists()
        self._set_up_by_unittest()
        assert not FILE_PATH_VACATE.target.exists()
        self._do_cleanups_by_unittest()
        assert not FILE_PATH_VACATE.target.exists()

    @staticmethod
    def _set_up_by_unittest():
        ConfigurableTestCase(methodName="setUp").setUp()

    @staticmethod
    def _do_cleanups_by_unittest():
        ConfigurableTestCase(methodName="doCleanups").doCleanups()


class TestAdvancedConfigHandlerUnittest(AdvancedConfigurableTestCase):
    """Tests for unittest scenario"""

    @staticmethod
    def test_advanced():
        """Config file for test should be loaded."""
        assert FILE_PATH_DEPLOY.target.read_text() == CONTENT_IN_PROJECT_TEST_RESOURCE_FILE_FOR_ADVANCED



"""This module implements file state dependencies."""
from abc import abstractmethod
from pathlib import Path

from fixturefilehandler.file_paths import RelativeDeployFilePath
from tests.testlibraries.parametrizer.checkers import ContentChecker, NotExistChecker


class FilePathStateHandler:
    """This class implements abstract file state dependency."""

    FILE_NAME_IN_DIRECTORY: Path = Path("file.txt")

    @classmethod
    @abstractmethod
    def create_checker(cls, path: Path):
        """This method returns checker."""

    @classmethod
    @abstractmethod
    def create_file(cls, file_path: RelativeDeployFilePath):
        """This method returns checker."""

    @classmethod
    @abstractmethod
    def create_directory(cls, file_path: RelativeDeployFilePath):
        """This method returns checker."""

    @classmethod
    def assert_file_state(cls, path: Path):
        """This method checks file state."""
        checker = cls.create_checker(path)
        checker.do_assertion()

    @classmethod
    def assert_directory_state(cls, path: Path):
        """This method checks file state."""
        checker = cls.create_checker(path / cls.FILE_NAME_IN_DIRECTORY)
        checker.do_assertion()


class NoExistHandler(FilePathStateHandler):
    """This class implements abstract file state dependency on case when not exist."""

    @classmethod
    def create_checker(cls, path: Path):
        return NotExistChecker(path)

    @classmethod
    def create_file(cls, file_path: RelativeDeployFilePath):
        pass

    @classmethod
    def create_directory(cls, file_path: RelativeDeployFilePath):
        pass

    @classmethod
    def assert_directory_state(cls, path: Path):
        """This method checks file state."""
        checker = cls.create_checker(path)
        checker.do_assertion()


class ContentExistingHandler(FilePathStateHandler):
    """This class implements abstract file state dependency on case when existing content."""

    CONTENT = "Content in existing file"

    @classmethod
    def create_checker(cls, path: Path):
        return ContentChecker(path, cls.CONTENT)

    @classmethod
    def create_file(cls, file_path: RelativeDeployFilePath):
        file_path.target.write_text(cls.CONTENT)

    @classmethod
    def create_directory(cls, file_path: RelativeDeployFilePath):
        file_path.target.mkdir()
        (file_path.target / cls.FILE_NAME_IN_DIRECTORY).write_text(cls.CONTENT)


class ContentBackupHandler(FilePathStateHandler):
    """This class implements abstract file state dependency on case when backup content."""

    CONTENT = "Content in backup file"

    @classmethod
    def create_checker(cls, path: Path):
        return ContentChecker(path, cls.CONTENT)

    @classmethod
    def create_file(cls, file_path: RelativeDeployFilePath):
        file_path.backup.write_text(cls.CONTENT)

    @classmethod
    def create_directory(cls, file_path: RelativeDeployFilePath):
        file_path.backup.mkdir()
        (file_path.backup / cls.FILE_NAME_IN_DIRECTORY).write_text(cls.CONTENT)


class ContentResourceHandler(FilePathStateHandler):
    """This class implements abstract file state dependency on case when resource content."""

    CONTENT = "Content in resource file"

    @classmethod
    def create_checker(cls, path: Path):
        return ContentChecker(path, cls.CONTENT)

    @classmethod
    def create_file(cls, file_path: RelativeDeployFilePath):
        file_path.resource.write_text(cls.CONTENT)

    @classmethod
    def create_directory(cls, file_path: RelativeDeployFilePath):
        file_path.resource.mkdir()
        (file_path.resource / cls.FILE_NAME_IN_DIRECTORY).write_text(cls.CONTENT)
